#version 150

#moj_import <fog.glsl>
#moj_import <noise.glsl>

uniform vec4 ColorModulator;
uniform float GameTime;
uniform float FogStart;
uniform float FogEnd;
uniform vec4 FogColor;
uniform mat4 ModelViewMat;
uniform mat4 ProjMat;
uniform vec2 ScreenSize;

in float vertexDistance;
in vec4 rawPos;

out vec4 fragColor;

vec3 sampleSky(vec3 v) {
    vec3 sky = ColorModulator.rgb;

    sky = mix(FogColor.rgb, sky, pow(clamp(v.y, 0.0, 1.1), 0.8));
    return sky + fract(dot(gl_FragCoord.xy - vec2(0.5, 0.5), vec2(1.0/16.0, 10.0/36.0) + 0.25)) / 256;
}

vec3 sampleNightSky(vec3 v) {
    vec3 sky = ColorModulator.rgb + pow(1.0 - v.y, 1.5) * FogColor.rgb;

    float noise1 = clamp((snoise(vec4(v * 100.0, 0.05 * sin(GameTime * 628.318))) - 0.92) * 100.0, -16.0, 1.0);
    float noise2 = clamp((snoise(vec4(v * vec3(-1.0, 1.0, -1.0) * 100.0, 0.05 * cos(GameTime * 628.318))) - 0.95) * 100.0, -16.0, 1.0);
    noise1 = max(noise1, (noise1 + 16.0) * 0.02);
    noise2 = max(noise2, (noise2 + 16.0) * 0.02);
    vec3 stars = noise1 * vec3(0.9, 0.9, 1.0);
    stars += noise2 * vec3(1.0, 0.95, 0.9);

    sky = mix(FogColor.rgb, max(sky, stars), pow(clamp(v.y, 0.0, 1.0), 0.5));
    return sky + fract(dot(gl_FragCoord.xy - vec2(0.5, 0.5), vec2(1.0/16.0, 10.0/36.0) + 0.25)) / 256;
}

void main() {
    if (ProjMat[3][2] != -2.0) {
        vec2 pos = gl_FragCoord.xy / ScreenSize;
        pos -= vec2(0.5, 0.5);
        pos *= 2;
        vec4 cast_pos = vec4(pos, 1.0, 1.0);
        cast_pos = inverse(ProjMat) * cast_pos;
        cast_pos = normalize(cast_pos);
        vec3 v = normalize(cast_pos.xyz * mat3(ModelViewMat));

        float brightness = length(ColorModulator.rgb * 4.0);
        fragColor = vec4(mix(sampleNightSky(v), sampleSky(v), clamp(brightness, 0.0, 1.0)), 1.0);
    }
    else {
        fragColor = linear_fog(ColorModulator, vertexDistance, FogStart, FogEnd, FogColor);
    }
}
