#version 150

#moj_import <light.glsl>
#moj_import <fog.glsl>

in vec3 Position;
in vec4 Color;
in vec2 UV0;
in ivec2 UV2;
in vec3 Normal;

uniform sampler2D Sampler0;
uniform sampler2D Sampler2;

uniform mat4 ModelViewMat;
uniform mat4 ProjMat;
uniform vec3 ChunkOffset;
uniform int FogShape;
uniform vec4 FogColor;
uniform float GameTime;

out float vertexDistance;
out vec4 vertexColor;
out vec4 vertexLight;
out vec2 texCoord0;
out vec4 normal;

mat3 rotateX(float theta) {
    float c = cos(theta);
    float s = sin(theta);
    return mat3(
        vec3(1, 0, 0),
        vec3(0, c, -s),
        vec3(0, s, c)
        );
}

mat3 rotateY(float theta) {
    float c = cos(theta);
    float s = sin(theta);
    return mat3(
        vec3(c, 0, s),
        vec3(0, 1, 0),
        vec3(-s, 0, c)
        );
}

mat3 rotateZ(float theta) {
    float c = cos(theta);
    float s = sin(theta);
    return mat3(
        vec3(c, -s, 0),
        vec3(s, c, 0),
        vec3(0, 0, 1)
        );
}

vec3 rotate(vec3 point, vec3 angle) {
    return point * rotateX(angle.x) * rotateY(angle.y) * rotateZ(angle.z);
}

void main() {
    vec3 pos = Position + ChunkOffset;

    vertexColor = Color;
    vertexLight = vec4(1.0);

    vec4 color = texture(Sampler0, UV0);
    if (int(color.a * 255.0 + 0.5) == 245) {
        vec3 blockPos = mod(Position, 1.0) - 0.5;
        pos -= blockPos;
        blockPos *= 1.0 + max(0.0, sin(GameTime * 20000.0) - 0.8) * 0.25;
        blockPos = rotate(blockPos, vec3(sin(GameTime * 1000.0) * 4.0, GameTime * 1000.0, cos(GameTime * 1000.0) * 4.0));
        pos += blockPos;
        pos.y += 0.0625;
    } 
    else if (ProjMat[3][2] != -2.0) {
        vertexLight = minecraft_sample_lightmap(Sampler2, UV2, FogColor);

        int control = int(texture(Sampler0, UV0).a * 255.0 + 0.5);
        float pi = 3.14159;
        vec3 wave = vec3(sin(Position.y * -(pi / 4.0) + Position.x * (pi / 8.0) + Position.z * (pi / 8.0) + GameTime * 2000.0),
         sin(Position.y * -(pi / 4.0) + Position.x * (pi / 4.0) - Position.z * (pi / 8.0) + GameTime * 1000.0),
         cos(Position.y * -(pi / 4.0) + Position.x * (pi / 4.0) + Position.z * (pi / 4.0) + GameTime * 1500.0));
        if (control == 2 || control == 252) {
            pos += wave * 0.025;
        }
        else if (control == 3 || control == 251) {
            pos += wave * 0.08;
        }
    }

    vertexDistance = fog_distance(ModelViewMat, pos, FogShape);
    gl_Position = ProjMat * ModelViewMat * vec4(pos, 1.0);

    texCoord0 = UV0;
    normal = ProjMat * ModelViewMat * vec4(Normal, 0.0);
}
