#version 150

#moj_import <light.glsl>
#moj_import <fog.glsl>

in vec3 Position;
in vec4 Color;
in vec2 UV0;
in ivec2 UV1;
in ivec2 UV2;
in vec3 Normal;

uniform sampler2D Sampler0;
uniform sampler2D Sampler1;
uniform sampler2D Sampler2;

uniform mat4 ModelViewMat;
uniform mat4 ProjMat;
uniform mat3 IViewRotMat;
uniform int FogShape;
uniform vec4 FogColor;
uniform float GameTime;

uniform vec3 Light0_Direction;
uniform vec3 Light1_Direction;

out float vertexDistance;
out vec4 vertexColor;
out vec4 lightMapColor;
out vec4 overlayColor;
out vec2 texCoord0;
out vec4 normal;
flat out int player;
out vec3 settings;
out vec3 settings2;
out float part;

#define SPACING 1024.0
#define MAXRANGE (0.5 * SPACING)

const vec4[] subuvs = vec4[](
    vec4(4.0,  0.0,  8.0,  4.0 ), // 4x4x12
    vec4(8.0,  0.0,  12.0, 4.0 ),
    vec4(0.0,  4.0,  4.0,  16.0),
    vec4(4.0,  4.0,  8.0,  16.0),
    vec4(8.0,  4.0,  12.0, 16.0), 
    vec4(12.0, 4.0,  16.0, 16.0), 
    vec4(4.0,  0.0,  7.0,  4.0 ), // 4x3x12
    vec4(7.0,  0.0,  10.0, 4.0 ),
    vec4(0.0,  4.0,  4.0,  16.0),
    vec4(4.0,  4.0,  7.0,  16.0),
    vec4(7.0,  4.0,  11.0, 16.0), 
    vec4(11.0, 4.0,  14.0, 16.0),
    vec4(4.0,  0.0,  12.0, 4.0 ), // 4x8x12
    vec4(12.0,  0.0, 20.0, 4.0 ),
    vec4(0.0,  4.0,  4.0,  16.0),
    vec4(4.0,  4.0,  12.0, 16.0),
    vec4(12.0, 4.0,  16.0, 16.0),
    vec4(16.0, 4.0,  24.0, 16.0)
    );

const vec2[] origins = vec2[](
    vec2(40.0, 16.0), // right arm
    vec2(40.0, 32.0),
    vec2(32.0, 48.0), // left arm
    vec2(48.0, 48.0),
    vec2(16.0, 16.0), // torso
    vec2(16.0, 32.0),
    vec2(0.0,  16.0), // right leg
    vec2(0.0,  32.0),
    vec2(16.0, 48.0), // left leg
    vec2(0.0,  48.0)
    );

float hash(float p) {
    p = fract(p * 0.011);
    p *= p + 7.5;
    p *= p + p;
    return fract(p);
}

float noise(float x) {
    float i = floor(x);
    float f = fract(x);
    float u = f * f * (3.0 - 2.0 * f);
    return mix(hash(i), hash(i + 1.0), u);
}

float fbm(float x) {
    float v = 0.0;
    float a = 0.5;
    float shift = float(100);
    for (int i = 0; i < 3; ++i) {
        v += a * noise(x);
        x = x * 2.0 + shift;
        a *= 0.5;
    }
    return v;
}

void main() {
    vec3 pos = IViewRotMat * Position;

    gl_Position = ProjMat * ModelViewMat * vec4(Position, 1.0);

    vec4 damageFlash = texelFetch(Sampler1, UV1, 0);
    overlayColor = damageFlash;
    texCoord0 = UV0;
    normal = ProjMat * ModelViewMat * vec4(Normal, 0.0);
    vertexColor = minecraft_mix_light(Light0_Direction, Light1_Direction, Normal, Color);

    if (ProjMat[3][2] == -2.0) {
        lightMapColor = vec4(1.0);
    }
    else {
        lightMapColor = minecraft_sample_lightmap(Sampler2, UV2, FogColor);

        ivec2 dim = textureSize(Sampler0, 0);

        if (dim.x != 64 || dim.y != 64) { // short circuit if cannot be player
            part = 0.0;
            texCoord0 = UV0;
            vertexDistance = fog_distance(ModelViewMat, IViewRotMat * Position, FogShape);
            gl_Position = ProjMat * ModelViewMat * vec4(Position, 1.0);
        }
        else {
            vec2 UVout = UV0;
            vec2 UVout2 = vec2(0.0);
            int partId = -int((pos.y - MAXRANGE) / SPACING);

            part = float(partId);

            if (partId == 0) { // higher precision position if no translation is needed
                gl_Position = ProjMat * ModelViewMat * vec4(Position, 1.0);
            }
            else {
                vec4 samp1 = texture(Sampler0, vec2(54.0 / 64.0, 20.0 / 64.0));
                vec4 samp2 = texture(Sampler0, vec2(55.0 / 64.0, 20.0 / 64.0));
                bool slim = samp1.a == 0.0 || (((samp1.r + samp1.g + samp1.b) == 0.0) && ((samp2.r + samp2.g + samp2.b) == 0.0) && samp1.a == 1.0 && samp2.a == 1.0);
                int outerLayer = (gl_VertexID / 24) % 2; 
                int faceId = (gl_VertexID % 24) / 4;
                int vertexId = gl_VertexID % 4;
                int subuvIndex = faceId;

                pos.y += SPACING * partId;
                gl_Position = ProjMat * ModelViewMat * vec4(inverse(IViewRotMat) * pos, 1.0);

                UVout = origins[2 * (partId - 1) + outerLayer];
                UVout2 = origins[2 * (partId - 1)];

                if (slim && (partId == 1 || partId == 2)) {
                    subuvIndex += 6;
                }
                else if (partId == 3) {
                    subuvIndex += 12;
                }

                vec4 subuv = subuvs[subuvIndex];

                vec2 offset = vec2(0.0);
                if (faceId == 1) {
                    if (vertexId == 0) {
                        offset += subuv.zw;
                    }
                    else if (vertexId == 1) {
                        offset += subuv.xw;
                    }
                    else if (vertexId == 2) {
                        offset += subuv.xy;
                    }
                    else {
                        offset += subuv.zy;
                    }
                }
                else {
                    if (vertexId == 0) {
                        offset += subuv.zy;
                    }
                    else if (vertexId == 1) {
                        offset += subuv.xy;
                    }
                    else if (vertexId == 2) {
                        offset += subuv.xw;
                    }
                    else {
                        offset += subuv.zw;
                    }
                }

                UVout += offset;
                UVout2 += offset;
                UVout /= 64.0;
                UVout2 /= 64.0;
            }

            vertexDistance = fog_distance(ModelViewMat, pos, FogShape);
            texCoord0 = UVout;
        }
    }

    player = 0;
    if (ivec2(textureSize(Sampler0, 0)) == ivec2(64, 64) && ivec4(texelFetch(Sampler0, ivec2(0, 16), 0) * 255.0 + 0.5) == ivec4(1, 1, 1, 255)) {
        player = 1;
        settings = texelFetch(Sampler0, ivec2(1, 16), 0).rgb;
        settings2 = texelFetch(Sampler0, ivec2(2, 16), 0).rgb;
        // Check if current vertex is on the player's face
        if (texCoord0.x >= 0.125 && texCoord0.y >= 0.125 && texCoord0.x <= 0.25 && texCoord0.y <= 0.25) {
            // Damage face
            if (settings.r > 0.99 && damageFlash != vec4(1.0)) {
                if (gl_VertexID == 12) {
                    texCoord0 = vec2(0.5, 0.0);
                }
                else if (gl_VertexID == 13) {
                    texCoord0 = vec2(0.375, 0.0);
                }
                else if (gl_VertexID == 14) {
                    texCoord0 = vec2(0.375, 0.125);
                }
                else if (gl_VertexID == 15) {
                    texCoord0 = vec2(0.5, 0.125);
                }
            // Blinking
        }
        else if (settings.b > 0.99) {
            vec3 blinkOffset = texelFetch(Sampler0, ivec2(20, 20), 0).rgb;
            if (fbm(floor(GameTime * 6400.0) * 64.0 + (blinkOffset.r + blinkOffset.g + blinkOffset.b) * 0.01) > 0.725) {
                if (gl_VertexID == 12) {
                    texCoord0 = vec2(0.125, 0.0);
                }
                else if (gl_VertexID == 13) {
                    texCoord0 = vec2(0.0, 0.0);
                }
                else if (gl_VertexID == 14) {
                    texCoord0 = vec2(0.0, 0.125);
                }
                else if (gl_VertexID == 15) {
                    texCoord0 = vec2(0.125, 0.125);
                }
            }
        }
    }
    else if (texCoord0.x >= 0.625 && texCoord0.y >= 0.125 && texCoord0.x <= 0.75 && texCoord0.y <= 0.25) {
        // Damage face
        if (settings.r > 0.99 && damageFlash != vec4(1.0)) {
            if (gl_VertexID == 156) {
                texCoord0 = vec2(1.0, 0.0);
            }
            else if (gl_VertexID == 157) {
                texCoord0 = vec2(0.875, 0.0);
            }
            else if (gl_VertexID == 158) {
                texCoord0 = vec2(0.875, 0.125);
            }
            else if (gl_VertexID == 159) {
                texCoord0 = vec2(1.0, 0.125);
            }
            // Blinking
        }
        else if (settings.b > 0.99) {
            vec3 blinkOffset = texelFetch(Sampler0, ivec2(3, 19), 0).rgb;
            if (fbm(floor(GameTime * 6400.0) * 64.0 + (blinkOffset.r + blinkOffset.g + blinkOffset.b) * 0.01) > 0.725) {
                if (gl_VertexID == 156) {
                    texCoord0 = vec2(0.625, 0.0);
                }
                else if (gl_VertexID == 157) {
                    texCoord0 = vec2(0.5, 0.0);
                }
                else if (gl_VertexID == 158) {
                    texCoord0 = vec2(0.5, 0.125);
                }
                else if (gl_VertexID == 159) {
                    texCoord0 = vec2(0.625, 0.125);
                }
            }
        }
    }
    // Custom damage flash
    if (settings.g > 0.99) {
        overlayColor = texelFetch(Sampler0, ivec2(0, 19), 0);
        overlayColor.a = damageFlash.a;
    }
    // Cape
    if (settings2.g > 0.99) {
        // Player
        if (part < 0.5) {
            vec3 normal = IViewRotMat * Normal;
            float wave = (sin(GameTime * 800.0) * 0.5 + 0.5) * 0.2;
            float angle = wave + 0.2;
            if (gl_VertexID == 284) {
                player = 2;
                pos += normal * 0.045;
                texCoord0 = vec2(1.0, 0.25);
            }
            else if (gl_VertexID == 285) {
                player = 2;
                pos += normal * 0.045;
                texCoord0 = vec2(0.875, 0.25);
            }
            else if (gl_VertexID == 286) {
                player = 2;
                texCoord0 = vec2(0.875, 0.5);
                pos += normal * angle;
                pos.y += length(angle) * 0.4 - 0.2;
            }
            else if (gl_VertexID == 287) {
                player = 2;
                texCoord0 = vec2(1.0, 0.5);
                pos += normal * angle;
                pos.y += length(angle) * 0.4 - 0.2;
            }
        }
        // Display
        else if (int(part + 0.5) == 3) {
            vec3 normal = IViewRotMat * Normal;
            float wave = (sin(GameTime * 800.0) * 0.5 + 0.5) * 0.2;
            float angle = wave + 0.2;
            if (gl_VertexID % 48 == 44) {
                player = 2;
                pos += normal * 0.045;
                texCoord0 = vec2(1.0, 0.25);
            }
            else if (gl_VertexID % 48 == 45) {
                player = 2;
                pos += normal * 0.045;
                texCoord0 = vec2(0.875, 0.25);
            }
            else if (gl_VertexID % 48 == 46) {
                player = 2;
                texCoord0 = vec2(0.875, 0.5);
                pos += normal * angle;
                pos.y += length(angle) * 0.4 - 0.2;
            }
            else if (gl_VertexID % 48 == 47) {
                player = 2;
                texCoord0 = vec2(1.0, 0.5);
                pos += normal * angle;
                pos.y += length(angle) * 0.4 - 0.2;
            }
        }
    }

    gl_Position = ProjMat * ModelViewMat * vec4(pos * IViewRotMat, 1.0);
}

vertexDistance = fog_distance(ModelViewMat, pos, FogShape);
}
