#version 150

#moj_import <light.glsl>
#moj_import <fog.glsl>

in vec3 Position;
in vec4 Color;
in vec2 UV0;
in ivec2 UV2;
in vec3 Normal;

uniform sampler2D Sampler0;
uniform sampler2D Sampler2;

uniform mat4 ModelViewMat;
uniform mat4 ProjMat;
uniform vec3 ChunkOffset;
uniform int FogShape;
uniform vec4 FogColor;
uniform float GameTime;

out float vertexDistance;
out vec4 vertexColor;
out vec4 vertexLight;
out vec2 texCoord0;
out vec4 normal;
out float material;
out float wave;
out float fade;
out float sky;

void main() {
    vec3 pos = Position + ChunkOffset;

    gl_Position = ProjMat * ModelViewMat * vec4(pos, 1.0);
    texCoord0 = UV0;

    sky = float(UV2.y) / 256.0;
    material = 0.0;
    if (ProjMat[3][2] == -2.0) {
        vertexColor = Color;
        vertexLight = vec4(1.0);
    }
    else {
        vertexColor = Color;
        vertexLight = minecraft_sample_lightmap(Sampler2, UV2, FogColor);
        if (!(Color.r == Color.g && Color.g == Color.b)) {
            material = 1.0;
            float pi = 3.14159;
            wave = sin(GameTime * 1600.0 + Position.x * pi / 8.0 + Position.z * pi / 2.0);
            wave += sin(GameTime * 1000.0 + Position.x * pi / 4.0 + Position.z * pi / 8.0) * 0.5;
            pos.y += (wave + sin(GameTime * 400.0)) * 0.02 - 0.02;
            gl_Position = ProjMat * ModelViewMat * vec4(pos, 1.0);
        }
        else if (int(texture(Sampler0, UV0).a * 255.0 + 0.5) == 247) {
            material = 2.0;
            vertexColor = Color;
            vec3 fadeOffset = (mod(Position, 1.0) - 0.5) / Normal;
            fade = max(fadeOffset.x, fadeOffset.z) * 2.0;
        }
    }

    vertexDistance = fog_distance(ModelViewMat, pos, FogShape);
    normal = ProjMat * ModelViewMat * vec4(Normal, 0.0);
}
