#version 150

#define MINECRAFT_LIGHT_POWER   (0.6)
#define MINECRAFT_AMBIENT_LIGHT (0.4)

vec4 minecraft_mix_light(vec3 lightDir0, vec3 lightDir1, vec3 normal, vec4 color) {
    lightDir0 = normalize(lightDir0);
    lightDir1 = normalize(lightDir1);
    float light0 = max(0.0, dot(lightDir0, normal));
    float light1 = max(0.0, dot(lightDir1, normal));
    float lightAccum = min(1.0, (light0 + light1) * MINECRAFT_LIGHT_POWER + MINECRAFT_AMBIENT_LIGHT);
    return vec4(color.rgb * lightAccum, color.a);
}

vec4 minecraft_sample_lightmap(sampler2D lightMap, ivec2 uv, vec4 fogColor) {
    vec3 light = texture(lightMap, vec2(0.5) / 16.0).rgb;
    light.r = pow(light.r, 2.0);
    light.g = pow(light.g, 2.0);
    light.b = pow(light.b, 1.5);
    //vec3 skyLight = texture(lightMap, vec2(0.5, 15.5) / 16.0).rgb;
    vec3 skyLight = vec3(length(fogColor.rgb * 0.5 + 0.1));

    light = max(light, vec3(0.9, 0.9, 0.9) * pow(clamp(pow(uv.x / 180.0, 1.5), 0.0, 1.1), 1.5));

    light = max(light, vec3(0.6, 0.6, 0.9) * pow(uv.y / 256.0, 3.0) * (skyLight.b * 1.8 - 0.4));
    light = max(light, mix(vec3(0.0, 0.0, 0.0), vec3(1.4, 1.4, 1.4), skyLight.r * 1.5 - 0.7) * pow(uv.y / 256.0 / 0.9 - 0.05, 16.0));

    return vec4(light, 1.0);
}
