#version 150

uniform sampler2D DiffuseSampler;
uniform sampler2D DataSampler;
uniform sampler2D GlowSampler;
uniform sampler2D HighlightsSampler;

uniform vec4 ColorModulate;

in vec2 texCoord;

out vec4 fragColor;

float decodeBrightness(vec3 brightness) {
    brightness = floor(brightness * 255.0 + 0.5) / 255.0;
    return brightness.r + (brightness.g / 255.0) + (brightness.b / (255.0 * 255.0));
}

vec3 rgb2hsv(vec3 c) {
    vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
    vec4 p = mix(vec4(c.bg, K.wz), vec4(c.gb, K.xy), step(c.b, c.g));
    vec4 q = mix(vec4(p.xyw, c.r), vec4(c.r, p.yzx), step(p.x, c.r));

    float d = q.x - min(q.w, q.y);
    float e = 1.0e-10;
    return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}


vec3 hsv2rgb(vec3 c) {
    vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
    vec3 p = abs(fract(c.xxx + K.xyz) * 6.0 - K.www);
    return c.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y);
}

float hueDistance(float hue, float targetHue) {
    return min(abs(hue - targetHue), min(abs(hue - 1.0 - targetHue), abs(hue + 1.0 - targetHue))) * 2.0;
}

void main(){
    float brightness = decodeBrightness(texture(DataSampler, vec2(0.0, 0.0)).rgb);
    vec4 color = texture(DiffuseSampler, texCoord);
    vec3 highlights = texture(HighlightsSampler, texCoord).rgb;
    color = max(color, mix(color, vec4(1.0), highlights.r));
    color.rgb *= 1.0 + (1.0 - brightness) * 0.5;
    color += texture(GlowSampler, texCoord) * (1.1 - brightness);

    //color.rgb = rgb2hsv(color.rgb);
    //color.g = clamp(color.g - (hueDistance(color.r, 0.3) * 2.0), 0.0, 1.0);
    //color.g = clamp(color.g - (hueDistance(color.r, 0.85) * 1.0), 0.0, 1.0);
    //color.rgb = hsv2rgb(color.rgb);

    fragColor = color * ColorModulate;
}
