#version 150

uniform sampler2D DiffuseSampler;
uniform sampler2D DiffuseDepthSampler;
uniform sampler2D ScreenSampler;

in vec2 texCoord;

out vec4 fragColor;

float decodeData(vec3 data) {
    return data.r + (data.g / 255.0) + (data.b / (255.0 * 255.0));
}

vec3 encodeData(float data) {
    return vec3(clamp(data, 0.0, 1.0), mod(data * 255.0 + 0.5, 1.0), mod(data * 255.0 * 255.0 + 0.5, 1.0));
}

void main(){
    if (gl_FragCoord.x < 1.5 && gl_FragCoord.y < 1.5) {
        float previousBrightness = decodeData(texelFetch(DiffuseSampler, ivec2(0, 0), 0).rgb);
        float brightness = 0.0;

        float maxBrightness = 0.0;
        for (float i = 0; i < 1.0; i += 0.05) {
            for (float j = 0; j < 1.0; j += 0.05) {
                maxBrightness += 1.0;
                vec3 screenColor = texture(ScreenSampler, vec2(i, j)).rgb;
                brightness += (screenColor.r + screenColor.g + screenColor.b) * 2.0;
            }
        }
        brightness /= maxBrightness;

        float brightnessDistance = brightness - previousBrightness;
        fragColor = vec4(encodeData(previousBrightness + brightnessDistance * 0.005), 1.0);
    }
    else if (gl_FragCoord.x < 2.5 && gl_FragCoord.y < 1.5) {
        float previousDOF = decodeData(texelFetch(DiffuseSampler, ivec2(1, 0), 0).rgb);
        float DOF = min(texture(DiffuseDepthSampler, vec2(0.5, 0.4)).r, texture(DiffuseDepthSampler, vec2(0.5, 0.6)).r);
        DOF = min(DOF, texture(DiffuseDepthSampler, vec2(0.4, 0.5)).r);
        DOF = min(DOF, texture(DiffuseDepthSampler, vec2(0.5, 0.5)).r);
        DOF = min(DOF, texture(DiffuseDepthSampler, vec2(0.6, 0.5)).r);
        float DOFDistance = DOF - previousDOF;
        fragColor = vec4(encodeData(previousDOF + DOFDistance * 0.1), 1.0);

    }
    else {
        fragColor = vec4(0.0);
    }
}
