#version 150

uniform sampler2D DiffuseSampler;
uniform sampler2D DiffuseDepthSampler;
uniform sampler2D TranslucentSampler;
uniform sampler2D TranslucentDepthSampler;
uniform sampler2D ItemEntitySampler;
uniform sampler2D ItemEntityDepthSampler;
uniform sampler2D ParticlesSampler;
uniform sampler2D ParticlesDepthSampler;
uniform sampler2D WeatherSampler;
uniform sampler2D WeatherDepthSampler;
uniform sampler2D CloudsSampler;
uniform sampler2D CloudsDepthSampler;

in vec2 texCoord;

float totalDepth = 1.0;

out vec4 fragColor;

float linearizeDepth(float depth, float far) {
    float near = 0.05;
    return (2.0 * near * far) / (far + near - depth * (far - near));    
}

float decodeDepth(vec3 depth) {
    return depth.r + (depth.g / 255.0) + (depth.b / (255.0 * 255.0));
}

vec3 encodeDepth(float depth) {
    return vec3(clamp(depth, 0.0, 1.0), mod(depth * 255.0 + 0.5, 1.0), mod(depth * 255.0 * 255.0 + 0.5, 1.0));
}

void try_insert(vec4 color, float depth) {
    if ( color.a == 0.0 ) {
        return;
    }

    if (depth < totalDepth) {
        totalDepth = depth;
    }
}


void main() {
    try_insert( texture( DiffuseSampler, texCoord ), texture( DiffuseDepthSampler, texCoord ).r );
    try_insert( texture( TranslucentSampler, texCoord ), texture( TranslucentDepthSampler, texCoord ).r );
    try_insert( texture( ItemEntitySampler, texCoord ), texture( ItemEntityDepthSampler, texCoord ).r );
    try_insert( texture( ParticlesSampler, texCoord ), texture( ParticlesDepthSampler, texCoord ).r );
    try_insert( texture( WeatherSampler, texCoord ), texture( WeatherDepthSampler, texCoord ).r );
    try_insert( texture( CloudsSampler, texCoord ), texture( CloudsDepthSampler, texCoord ).r );

    float far = 16.0 * 16.0;
    //if (texture(DiffuseDepthSampler, vec2(0.0, 0.0)).r < 0.001) {
    //    far = texture(DiffuseSampler, vec2(0.0, 0.0)).r * 16.0 * 16.0;
    //}

    fragColor = vec4(encodeDepth(linearizeDepth(totalDepth, far) / 512.0), 1.0);
}
