#version 150

uniform sampler2D DiffuseSampler;
uniform sampler2D DiffuseDepthSampler;
uniform sampler2D BlurSampler;
uniform sampler2D DataSampler;

uniform vec2 OutSize;
uniform vec4 ColorModulate;

in vec2 texCoord;

out vec4 fragColor;

float decodeData(vec3 data) {
    return data.r + (data.g / 255.0) + (data.b / (255.0 * 255.0));
}

void main(){
    vec2 oneTex = 1.0 / OutSize;
    vec3 color = texture(DiffuseSampler, texCoord).rgb;
    vec3 blurredColor = texture(BlurSampler, texCoord).rgb;

    float depth = decodeData(texture(DiffuseDepthSampler, texCoord).rgb);
    if (depth < 0.999) {
        float DOF = decodeData(texelFetch(DataSampler, ivec2(1, 0), 0).rgb) * 0.64;
        
        color = mix(color, blurredColor, clamp(abs(depth - DOF) * 2.0, 0.0, 1.0));
    }

    fragColor = vec4(color, 1.0) * ColorModulate;
}
