#version 150

uniform sampler2D DiffuseSampler;
uniform sampler2D DiffuseDepthSampler;

uniform vec2 OutSize;
uniform vec4 ColorModulate;

in vec2 texCoord;

out vec4 fragColor;

float decodeDepth(vec3 depth) {
    return depth.r + (depth.g / 255.0) + (depth.b / (255.0 * 255.0));
}

void main(){
    vec2 oneTex = 1.0 / OutSize;
    vec3 color = vec3(0.0);

    vec3 diffuse = texture(DiffuseSampler, texCoord).rgb;
    float depth = decodeDepth(texture(DiffuseDepthSampler, texCoord).rgb);
    float successfulSamples = -1.0;
    for (float i = -1.0; i < 1.0; i += 0.25) {
        for (float j = -1.0; j < 1.0; j += 0.25) {
            if (length(vec2(i, j)) < 1.0) {
                vec2 uv = texCoord + vec2(i, j) * oneTex * 4.0 * (1.0 - pow(depth, 2.0));
                float diffuseDiff = clamp(distance(diffuse, texture(DiffuseSampler, uv).rgb) * 2.0, 0.0, 1.0);
                float depthDiff = clamp((decodeDepth(texture(DiffuseDepthSampler, uv).rgb) - depth - 0.02) * 1024.0, 0.0, 1.0);
                float highlight = diffuseDiff * depthDiff * 0.5;
                if (highlight > 0.001) {
                    color.r = max(color.r, highlight);
                    successfulSamples += 1.0;
                }
            }
        }
    }
    color.r *= clamp(successfulSamples * 0.1, 0.0, 0.5);

    fragColor = vec4(color, 1.0) * ColorModulate;
}
